@@ -146,14 +146,20 @@ class Serializer(base.Serializer):
                         self.xml.endElement("natural")
                     self.xml.endElement("object")
 
+                def queryset_iterator(obj, field):
+                    return getattr(obj, field.name).iterator()
+
             else:
 
                 def handle_m2m(value):
                     self.xml.addQuickElement("object", attrs={"pk": str(value.pk)})
 
+                def queryset_iterator(obj, field):
+                    return getattr(obj, field.name).only("pk").iterator()
+
             m2m_iter = getattr(obj, "_prefetched_objects_cache", {}).get(
                 field.name,
-                getattr(obj, field.name).iterator(),
+                queryset_iterator(obj, field),
             )
             for relobj in m2m_iter:
                 handle_m2m(relobj)
