@@ -14,6 +14,11 @@ from django.db import (
 from django.db.backends.base.base import BaseDatabaseWrapper
 from django.test import TestCase, override_settings
 
+try:
+    from django.db.backends.postgresql.psycopg_any import errors, is_psycopg3
+except ImportError:
+    is_psycopg3 = False
+
 
 @unittest.skipUnless(connection.vendor == "postgresql", "PostgreSQL tests")
 class Tests(TestCase):
@@ -164,7 +169,7 @@ class Tests(TestCase):
         settings["NAME"] = None
         settings["OPTIONS"] = {"service": "django_test"}
         params = DatabaseWrapper(settings).get_connection_params()
-        self.assertEqual(params["database"], "postgres")
+        self.assertEqual(params["dbname"], "postgres")
         self.assertNotIn("service", params)
 
     def test_connect_and_rollback(self):
@@ -223,21 +228,89 @@ class Tests(TestCase):
         The transaction level can be configured with
         DATABASES ['OPTIONS']['isolation_level'].
         """
-        from psycopg2.extensions import ISOLATION_LEVEL_SERIALIZABLE as serializable
+        from django.db.backends.postgresql.psycopg_any import IsolationLevel
 
         # Since this is a django.test.TestCase, a transaction is in progress
         # and the isolation level isn't reported as 0. This test assumes that
         # PostgreSQL is configured with the default isolation level.
-        # Check the level on the psycopg2 connection, not the Django wrapper.
+        # Check the level on the psycopg connection, not the Django wrapper.
         self.assertIsNone(connection.connection.isolation_level)
 
         new_connection = connection.copy()
-        new_connection.settings_dict["OPTIONS"]["isolation_level"] = serializable
+        new_connection.settings_dict["OPTIONS"][
+            "isolation_level"
+        ] = IsolationLevel.SERIALIZABLE
         try:
             # Start a transaction so the isolation level isn't reported as 0.
             new_connection.set_autocommit(False)
-            # Check the level on the psycopg2 connection, not the Django wrapper.
-            self.assertEqual(new_connection.connection.isolation_level, serializable)
+            # Check the level on the psycopg connection, not the Django wrapper.
+            self.assertEqual(
+                new_connection.connection.isolation_level,
+                IsolationLevel.SERIALIZABLE,
+            )
+        finally:
+            new_connection.close()
+
+    def test_connect_invalid_isolation_level(self):
+        self.assertIsNone(connection.connection.isolation_level)
+        new_connection = connection.copy()
+        new_connection.settings_dict["OPTIONS"]["isolation_level"] = -1
+        msg = (
+            "Invalid transaction isolation level -1 specified. Use one of the "
+            "psycopg.IsolationLevel values."
+        )
+        with self.assertRaisesMessage(ImproperlyConfigured, msg):
+            new_connection.ensure_connection()
+
+    def test_connect_role(self):
+        """
+        The session role can be configured with DATABASES
+        ["OPTIONS"]["assume_role"].
+        """
+        try:
+            custom_role = "django_nonexistent_role"
+            new_connection = connection.copy()
+            new_connection.settings_dict["OPTIONS"]["assume_role"] = custom_role
+            msg = f'role "{custom_role}" does not exist'
+            with self.assertRaisesMessage(errors.InvalidParameterValue, msg):
+                new_connection.connect()
+        finally:
+            new_connection.close()
+
+    @unittest.skipUnless(is_psycopg3, "psycopg3 specific test")
+    def test_connect_server_side_binding(self):
+        """
+        The server-side parameters binding role can be enabled with DATABASES
+        ["OPTIONS"]["server_side_binding"].
+        """
+        from django.db.backends.postgresql.base import ServerBindingCursor
+
+        new_connection = connection.copy()
+        new_connection.settings_dict["OPTIONS"]["server_side_binding"] = True
+        try:
+            new_connection.connect()
+            self.assertEqual(
+                new_connection.connection.cursor_factory,
+                ServerBindingCursor,
+            )
+        finally:
+            new_connection.close()
+
+    def test_connect_custom_cursor_factory(self):
+        """
+        A custom cursor factory can be configured with DATABASES["options"]
+        ["cursor_factory"].
+        """
+        from django.db.backends.postgresql.base import Cursor
+
+        class MyCursor(Cursor):
+            pass
+
+        new_connection = connection.copy()
+        new_connection.settings_dict["OPTIONS"]["cursor_factory"] = MyCursor
+        try:
+            new_connection.connect()
+            self.assertEqual(new_connection.connection.cursor_factory, MyCursor)
         finally:
             new_connection.close()
 
@@ -250,9 +323,21 @@ class Tests(TestCase):
         finally:
             new_connection.close()
 
+    def test_client_encoding_utf8_enforce(self):
+        new_connection = connection.copy()
+        new_connection.settings_dict["OPTIONS"]["client_encoding"] = "iso-8859-2"
+        try:
+            new_connection.connect()
+            if is_psycopg3:
+                self.assertEqual(new_connection.connection.info.encoding, "utf-8")
+            else:
+                self.assertEqual(new_connection.connection.encoding, "UTF8")
+        finally:
+            new_connection.close()
+
     def _select(self, val):
         with connection.cursor() as cursor:
-            cursor.execute("SELECT %s", (val,))
+            cursor.execute("SELECT %s::text[]", (val,))
             return cursor.fetchone()[0]
 
     def test_select_ascii_array(self):
@@ -283,6 +368,7 @@ class Tests(TestCase):
         for lookup in lookups:
             with self.subTest(lookup=lookup):
                 self.assertIn("::text", do.lookup_cast(lookup))
+        # RemovedInDjango51Warning.
         for lookup in lookups:
             for field_type in ("CICharField", "CIEmailField", "CITextField"):
                 with self.subTest(lookup=lookup, field_type=field_type):
@@ -290,16 +376,19 @@ class Tests(TestCase):
                         "::citext", do.lookup_cast(lookup, internal_type=field_type)
                     )
 
-    def test_correct_extraction_psycopg2_version(self):
-        from django.db.backends.postgresql.base import psycopg2_version
+    def test_correct_extraction_psycopg_version(self):
+        from django.db.backends.postgresql.base import Database, psycopg_version
 
-        with mock.patch("psycopg2.__version__", "4.2.1 (dt dec pq3 ext lo64)"):
-            self.assertEqual(psycopg2_version(), (4, 2, 1))
-        with mock.patch("psycopg2.__version__", "4.2b0.dev1 (dt dec pq3 ext lo64)"):
-            self.assertEqual(psycopg2_version(), (4, 2))
+        with mock.patch.object(Database, "__version__", "4.2.1 (dt dec pq3 ext lo64)"):
+            self.assertEqual(psycopg_version(), (4, 2, 1))
+        with mock.patch.object(
+            Database, "__version__", "4.2b0.dev1 (dt dec pq3 ext lo64)"
+        ):
+            self.assertEqual(psycopg_version(), (4, 2))
 
     @override_settings(DEBUG=True)
-    def test_copy_cursors(self):
+    @unittest.skipIf(is_psycopg3, "psycopg2 specific test")
+    def test_copy_to_expert_cursors(self):
         out = StringIO()
         copy_expert_sql = "COPY django_session TO STDOUT (FORMAT CSV, HEADER)"
         with connection.cursor() as cursor:
@@ -310,14 +399,34 @@ class Tests(TestCase):
             [copy_expert_sql, "COPY django_session TO STDOUT"],
         )
 
+    @override_settings(DEBUG=True)
+    @unittest.skipUnless(is_psycopg3, "psycopg3 specific test")
+    def test_copy_cursors(self):
+        copy_sql = "COPY django_session TO STDOUT (FORMAT CSV, HEADER)"
+        with connection.cursor() as cursor:
+            with cursor.copy(copy_sql) as copy:
+                for row in copy:
+                    pass
+        self.assertEqual([q["sql"] for q in connection.queries], [copy_sql])
+
     def test_get_database_version(self):
         new_connection = connection.copy()
         new_connection.pg_version = 110009
         self.assertEqual(new_connection.get_database_version(), (11, 9))
 
-    @mock.patch.object(connection, "get_database_version", return_value=(10,))
+    @mock.patch.object(connection, "get_database_version", return_value=(11,))
     def test_check_database_version_supported(self, mocked_get_database_version):
-        msg = "PostgreSQL 11 or later is required (found 10)."
+        msg = "PostgreSQL 12 or later is required (found 11)."
         with self.assertRaisesMessage(NotSupportedError, msg):
             connection.check_database_version_supported()
         self.assertTrue(mocked_get_database_version.called)
+
+    def test_compose_sql_when_no_connection(self):
+        new_connection = connection.copy()
+        try:
+            self.assertEqual(
+                new_connection.ops.compose_sql("SELECT %s", ["test"]),
+                "SELECT 'test'",
+            )
+        finally:
+            new_connection.close()
