@@ -10,8 +10,9 @@ class FeaturesTests(TestCase):
         if hasattr(connection.features, "supports_json_field"):
             del connection.features.supports_json_field
         msg = "unable to open database file"
-        with mock.patch(
-            "django.db.backends.base.base.BaseDatabaseWrapper.cursor",
+        with mock.patch.object(
+            connection,
+            "cursor",
             side_effect=OperationalError(msg),
         ):
             with self.assertRaisesMessage(OperationalError, msg):
