@@ -19,19 +19,6 @@ class Product(models.Model):
                 check=models.Q(price__gt=0),
                 name="%(app_label)s_%(class)s_price_gt_0",
             ),
-            models.CheckConstraint(
-                check=models.expressions.RawSQL(
-                    "price < %s", (1000,), output_field=models.BooleanField()
-                ),
-                name="%(app_label)s_price_lt_1000_raw",
-            ),
-            models.CheckConstraint(
-                check=models.expressions.ExpressionWrapper(
-                    models.Q(price__gt=500) | models.Q(price__lt=500),
-                    output_field=models.BooleanField(),
-                ),
-                name="%(app_label)s_price_neq_500_wrap",
-            ),
             models.CheckConstraint(
                 check=models.Q(
                     models.Q(unit__isnull=True) | models.Q(unit__in=["μg/mL", "ng/mL"])
@@ -51,6 +38,10 @@ class UniqueConstraintProduct(models.Model):
         ]
 
 
+class ChildUniqueConstraintProduct(UniqueConstraintProduct):
+    pass
+
+
 class UniqueConstraintConditionProduct(models.Model):
     name = models.CharField(max_length=255)
     color = models.CharField(max_length=32, null=True)
