@@ -19,7 +19,7 @@ from django.db.models import (
     Value,
     When,
 )
-from django.db.models.functions import Cast, Substr
+from django.db.models.functions import Abs, Cast, Length, Substr
 from django.db.models.lookups import (
     Exact,
     GreaterThan,
@@ -29,7 +29,7 @@ from django.db.models.lookups import (
     LessThanOrEqual,
 )
 from django.test import TestCase, skipUnlessDBFeature
-from django.test.utils import isolate_apps
+from django.test.utils import isolate_apps, register_lookup
 
 from .models import (
     Article,
@@ -130,7 +130,7 @@ class LookupTests(TestCase):
         # returns results using database-level iteration.
         self.assertIsInstance(Article.objects.iterator(), collections.abc.Iterator)
 
-        self.assertQuerysetEqual(
+        self.assertQuerySetEqual(
             Article.objects.iterator(),
             [
                 "Article 5",
@@ -144,7 +144,7 @@ class LookupTests(TestCase):
             transform=attrgetter("headline"),
         )
         # iterator() can be used on any QuerySet.
-        self.assertQuerysetEqual(
+        self.assertQuerySetEqual(
             Article.objects.filter(headline__endswith="4").iterator(),
             ["Article 4"],
             transform=attrgetter("headline"),
@@ -690,18 +690,18 @@ class LookupTests(TestCase):
 
     def test_none(self):
         # none() returns a QuerySet that behaves like any other QuerySet object
-        self.assertQuerysetEqual(Article.objects.none(), [])
-        self.assertQuerysetEqual(
+        self.assertSequenceEqual(Article.objects.none(), [])
+        self.assertSequenceEqual(
             Article.objects.none().filter(headline__startswith="Article"), []
         )
-        self.assertQuerysetEqual(
+        self.assertSequenceEqual(
             Article.objects.filter(headline__startswith="Article").none(), []
         )
         self.assertEqual(Article.objects.none().count(), 0)
         self.assertEqual(
             Article.objects.none().update(headline="This should not take effect"), 0
         )
-        self.assertQuerysetEqual(Article.objects.none().iterator(), [])
+        self.assertSequenceEqual(list(Article.objects.none().iterator()), [])
 
     def test_in(self):
         self.assertSequenceEqual(
@@ -784,15 +784,56 @@ class LookupTests(TestCase):
         ):
             Article.objects.filter(pub_date__gobbledygook="blahblah")
 
+    def test_unsupported_lookups_custom_lookups(self):
+        slug_field = Article._meta.get_field("slug")
+        msg = (
+            "Unsupported lookup 'lengtp' for SlugField or join on the field not "
+            "permitted, perhaps you meant length?"
+        )
+        with self.assertRaisesMessage(FieldError, msg):
+            with register_lookup(slug_field, Length):
+                Article.objects.filter(slug__lengtp=20)
+
     def test_relation_nested_lookup_error(self):
         # An invalid nested lookup on a related field raises a useful error.
-        msg = "Related Field got invalid lookup: editor"
+        msg = (
+            "Unsupported lookup 'editor' for ForeignKey or join on the field not "
+            "permitted."
+        )
         with self.assertRaisesMessage(FieldError, msg):
             Article.objects.filter(author__editor__name="James")
-        msg = "Related Field got invalid lookup: foo"
+        msg = (
+            "Unsupported lookup 'foo' for ForeignKey or join on the field not "
+            "permitted."
+        )
         with self.assertRaisesMessage(FieldError, msg):
             Tag.objects.filter(articles__foo="bar")
 
+    def test_unsupported_lookup_reverse_foreign_key(self):
+        msg = (
+            "Unsupported lookup 'title' for ManyToOneRel or join on the field not "
+            "permitted."
+        )
+        with self.assertRaisesMessage(FieldError, msg):
+            Author.objects.filter(article__title="Article 1")
+
+    def test_unsupported_lookup_reverse_foreign_key_custom_lookups(self):
+        msg = (
+            "Unsupported lookup 'abspl' for ManyToOneRel or join on the field not "
+            "permitted, perhaps you meant abspk?"
+        )
+        fk_field = Article._meta.get_field("author")
+        with self.assertRaisesMessage(FieldError, msg):
+            with register_lookup(fk_field, Abs, lookup_name="abspk"):
+                Author.objects.filter(article__abspl=2)
+
+    def test_filter_by_reverse_related_field_transform(self):
+        fk_field = Article._meta.get_field("author")
+        with register_lookup(fk_field, Abs):
+            self.assertSequenceEqual(
+                Author.objects.filter(article__abs=self.a1.pk), [self.au1]
+            )
+
     def test_regex(self):
         # Create some articles with a bit more interesting headlines for
         # testing field lookups.
@@ -812,52 +853,52 @@ class LookupTests(TestCase):
             ]
         )
         # zero-or-more
-        self.assertQuerysetEqual(
+        self.assertQuerySetEqual(
             Article.objects.filter(headline__regex=r"fo*"),
             Article.objects.filter(headline__in=["f", "fo", "foo", "fooo"]),
         )
-        self.assertQuerysetEqual(
+        self.assertQuerySetEqual(
             Article.objects.filter(headline__iregex=r"fo*"),
             Article.objects.filter(headline__in=["f", "fo", "foo", "fooo", "hey-Foo"]),
         )
         # one-or-more
-        self.assertQuerysetEqual(
+        self.assertQuerySetEqual(
             Article.objects.filter(headline__regex=r"fo+"),
             Article.objects.filter(headline__in=["fo", "foo", "fooo"]),
         )
         # wildcard
-        self.assertQuerysetEqual(
+        self.assertQuerySetEqual(
             Article.objects.filter(headline__regex=r"fooo?"),
             Article.objects.filter(headline__in=["foo", "fooo"]),
         )
         # leading anchor
-        self.assertQuerysetEqual(
+        self.assertQuerySetEqual(
             Article.objects.filter(headline__regex=r"^b"),
             Article.objects.filter(headline__in=["bar", "baxZ", "baz"]),
         )
-        self.assertQuerysetEqual(
+        self.assertQuerySetEqual(
             Article.objects.filter(headline__iregex=r"^a"),
             Article.objects.filter(headline="AbBa"),
         )
         # trailing anchor
-        self.assertQuerysetEqual(
+        self.assertQuerySetEqual(
             Article.objects.filter(headline__regex=r"z$"),
             Article.objects.filter(headline="baz"),
         )
-        self.assertQuerysetEqual(
+        self.assertQuerySetEqual(
             Article.objects.filter(headline__iregex=r"z$"),
             Article.objects.filter(headline__in=["baxZ", "baz"]),
         )
         # character sets
-        self.assertQuerysetEqual(
+        self.assertQuerySetEqual(
             Article.objects.filter(headline__regex=r"ba[rz]"),
             Article.objects.filter(headline__in=["bar", "baz"]),
         )
-        self.assertQuerysetEqual(
+        self.assertQuerySetEqual(
             Article.objects.filter(headline__regex=r"ba.[RxZ]"),
             Article.objects.filter(headline="baxZ"),
         )
-        self.assertQuerysetEqual(
+        self.assertQuerySetEqual(
             Article.objects.filter(headline__iregex=r"ba[RxZ]"),
             Article.objects.filter(headline__in=["bar", "baxZ", "baz"]),
         )
@@ -876,7 +917,7 @@ class LookupTests(TestCase):
         )
 
         # alternation
-        self.assertQuerysetEqual(
+        self.assertQuerySetEqual(
             Article.objects.filter(headline__regex=r"oo(f|b)"),
             Article.objects.filter(
                 headline__in=[
@@ -887,7 +928,7 @@ class LookupTests(TestCase):
                 ]
             ),
         )
-        self.assertQuerysetEqual(
+        self.assertQuerySetEqual(
             Article.objects.filter(headline__iregex=r"oo(f|b)"),
             Article.objects.filter(
                 headline__in=[
@@ -899,13 +940,13 @@ class LookupTests(TestCase):
                 ]
             ),
         )
-        self.assertQuerysetEqual(
+        self.assertQuerySetEqual(
             Article.objects.filter(headline__regex=r"^foo(f|b)"),
             Article.objects.filter(headline__in=["foobar", "foobarbaz", "foobaz"]),
         )
 
         # greedy matching
-        self.assertQuerysetEqual(
+        self.assertQuerySetEqual(
             Article.objects.filter(headline__regex=r"b.*az"),
             Article.objects.filter(
                 headline__in=[
@@ -917,7 +958,7 @@ class LookupTests(TestCase):
                 ]
             ),
         )
-        self.assertQuerysetEqual(
+        self.assertQuerySetEqual(
             Article.objects.filter(headline__iregex=r"b.*ar"),
             Article.objects.filter(
                 headline__in=[
@@ -945,7 +986,7 @@ class LookupTests(TestCase):
                 Article(pub_date=now, headline="bazbaRFOO"),
             ]
         )
-        self.assertQuerysetEqual(
+        self.assertQuerySetEqual(
             Article.objects.filter(headline__regex=r"b(.).*b\1").values_list(
                 "headline", flat=True
             ),
@@ -957,14 +998,14 @@ class LookupTests(TestCase):
         A regex lookup does not fail on null/None values
         """
         Season.objects.create(year=2012, gt=None)
-        self.assertQuerysetEqual(Season.objects.filter(gt__regex=r"^$"), [])
+        self.assertQuerySetEqual(Season.objects.filter(gt__regex=r"^$"), [])
 
     def test_regex_non_string(self):
         """
         A regex lookup does not fail on non-string fields
         """
         s = Season.objects.create(year=2013, gt=444)
-        self.assertQuerysetEqual(Season.objects.filter(gt__regex=r"^444$"), [s])
+        self.assertQuerySetEqual(Season.objects.filter(gt__regex=r"^444$"), [s])
 
     def test_regex_non_ascii(self):
         """
@@ -1212,9 +1253,7 @@ class LookupTests(TestCase):
 
     def test_exact_exists(self):
         qs = Article.objects.filter(pk=OuterRef("pk"))
-        seasons = Season.objects.annotate(
-            pk_exists=Exists(qs),
-        ).filter(
+        seasons = Season.objects.annotate(pk_exists=Exists(qs)).filter(
             pk_exists=Exists(qs),
         )
         self.assertCountEqual(seasons, Season.objects.all())
